/* ************************************************************************ */
/* DSERVER.C

   This example is the client side of a simple TLI example.  In this example,
   a simple message string is sent by the client, received by the server,
   and sent back to the client.  The client compares the information sent
   back, and then terminates.

   This example can be found in  the NetWare Programmers Guide for C, under
   the TLI Services section.
*/
/* ************************************************************************ */

#include <os2.h>

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <fcntl.h>

#define TLIINT  short
#define TLIUINT unsigned short

#define NWOS2
#define IS32BIT   /* NW library define for 32 bit OS/2 apps. */
#define BCPP      /* Defines use of Borland C++ compiler */

#include <nwcaldef.h>
#include <tispxipx.h>
#include <tiuser.h>

#define SPX_SOCKET 4646                   /* arbitrary socket */

void SPXDisconReason( TLIINT fd );

int flags;
struct t_bind tbind;
struct t_call tcall;
char NWPTR xportEP;
struct t_info NWPTR infoEP;

void main ( )
{
	IPX_ADDR spx_addr;
	SPX_OPTS spx_options;
	struct t_discon discon;
	char buf[ 100 ];
	TLIINT fd, nfd, len, c, listenResult;

	/* Step 1 */
	xportEP = "/dev/nspx";
	infoEP = NULL;
	if ( ( fd = t_open( xportEP, O_RDWR | O_NDELAY, infoEP ) ) == -1 )
	{
		t_error( "open of /dev/nspx failed" );
		exit( 1 );
	}

	/* Step 2 */
	spx_addr.ipxa_socket[ 0 ] = 0;
	spx_addr.ipxa_socket[ 1 ] = SPX_SOCKET;
	tbind.addr.len = sizeof( spx_addr );
	tbind.addr.maxlen = sizeof( spx_addr );
	tbind.addr.buf = ( char * ) &spx_addr;
	tbind.qlen = 1;

	if ( t_bind( fd, &tbind, &tbind ) == -1 )
	{
		t_error( "bind failed" );
		exit( 1 );
	}

	/* Step 3 */
	tcall.addr.buf = ( char * ) &spx_addr;
	tcall.addr.maxlen = sizeof( spx_addr );
	tcall.addr.len = sizeof( spx_addr );
	spx_options.spx_connectionID[ 0 ] = 0;
	spx_options.spx_connectionID[ 1 ] = 0;
	spx_options.spx_allocationNumber[ 0 ] = 0;
	spx_options.spx_allocationNumber[ 1 ] = 0;
	tcall.opt.buf = ( char * )&spx_options;
	tcall.opt.len = sizeof( spx_options );
	tcall.opt.maxlen = sizeof( spx_options );
	tcall.udata.buf = ( char * )0;
	tcall.udata.maxlen = 0;
	tcall.udata.len = 0;

	/* Step 4 */
	do
	{
	    do
	    {
		if ( kbhit( ) )
		{
			getch( ); /* clean up keyboard buffer */
			printf( "DSERVER terminated\n" );
			t_close( fd );
			exit( 0 );
		}
	    }
	    while ( ( listenResult = t_listen( fd, &tcall ) ) == -1 && t_errno == TNODATA );

	    /* Step 5 */
	    if ( listenResult != -1 )
	    {
		if ((nfd = t_open( "/dev/nspx",O_RDWR,(struct t_info * )0))==-1)
		{
			t_error( "t_open after listen failed" );
			t_close( fd );
			exit( 1 );
		}
		if (t_bind( nfd,(struct t_bind * )0,(struct t_bind * )0) == -1 )
		{
			t_error( "t_bind failed" );
			t_close( nfd );
			t_close( fd );
			exit( 1 );
		}
		if ( t_accept( fd, nfd, &tcall ) == -1 )
		{
			t_error( "t_accept failed" );
			t_close( nfd );
			t_close( fd );
			exit( 1 );
		}
		printf( "Connection accepted.\n" );
		flags = 0;                      /* initialize for t_rcv call */

		/* Step 6 */
		while ( ( len = t_rcv( nfd, buf, sizeof( buf), &flags ) ) != -1)
		{
			printf( "Received message '%s'\n", buf );
			if ( t_snd( nfd, buf, len, 0 ) == -1 )
			{
				t_error( "t_snd failed" );
				t_close( nfd );
				t_close( fd );
				exit( 1 );
			}
			printf( "Returned message '%s'\n", buf );
		}

			/* Step 7 */
		printf("t_errno = %d, t_look() = %d\n",t_errno , t_look( nfd ));
		if ( t_errno == TLOOK  &&  t_look( nfd ) == T_DISCONNECT )
			SPXDisconReason( nfd );
		else            /* some other error so disconnect */
		{
			if ( t_snddis( nfd, ( struct t_call * )0 ) == -1 )
				t_error( "t_snddis failed " );
		}
		t_close( nfd );
		printf( "listening...\n" );
	    } /* End of if (listenResult != -1 ) */
	}
	while ( listenResult != -1 );   /* End of outer do-while loop */

	/* Step 8 */
	if ( t_errno == TLOOK && t_look( fd ) == T_DISCONNECT )
		SPXDisconReason( fd );
	else
		t_error( "t_listen failed" );
	t_close( fd );
} /* main */

void SPXDisconReason( TLIINT fd )
{
	struct  t_discon discon;
	char buf[ 100 ];
	char *msg;

	discon.udata.buf = buf;
	discon.udata.len = 0;
	discon.udata.maxlen = sizeof(buf);

	printf( "SPX Connection terminated\n" );
	if ( t_rcvdis( fd, &discon ) == -1 )
	{
		t_error( "t_rcvdis failed" );
		exit( 2 );
	}
	printf( "Reason = %d\n" , discon.reason );
	switch( discon.reason )
	{
	case TLI_SPX_CONNECTION_FAILED:
		msg = "Connection failed";
		break;
	case TLI_SPX_CONNECTION_TERMINATED:
		msg = "Connection terminated by client";
		break;
	case TLI_SPX_MALFORMED_PACKET:
		msg = "Internal SPX TLIINTerface error -- malformed packet";
		break;
	default:
		msg = "Unknown termination reason";
	}
	printf( "SPX Connection terminated: %s\n", msg );
} /* SPXDisconReason */

